﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace JeuxDeLaVie
{
    public partial class Form1 : Form
    {

        /********************************************
         * Constants
         ********************************************/

        /*
         * number of rows in the world
         */
        const int worldHeight = 50;

        /*
         * number of columns in the world
         */ 
        const int worldWidth = 50;

        /*
         * Each case of the world is either false (no cell) or true (cell alive)
         */
        bool[,] world;

        /*
         * Size of a cell in pixels for drawing
         */ 
        const int cellWidth = 10;


        /********************************************
         * Creation
         ********************************************/

        public Form1()
        {
            InitializeComponent();
        }

        /*
         * Initialization is performed here
         */ 
        private void Form1_Load(object sender, EventArgs e)
        {
            world = new bool[worldHeight, worldWidth];
            InitialiseFormSize();
        }

        /*
       * Adjust components and form size according to world size
       */
        private void InitialiseFormSize()
        {
            panel1.Size = new Size(worldWidth * cellWidth + 1, worldHeight * cellWidth + 1);
            this.Size = new Size(worldWidth * cellWidth + 11, worldHeight * cellWidth + 75); // hugly constants for borders
        }


        /********************************************
         * Events
         ********************************************/

        /*
        * User click on Next Button
        */
        private void buttonNext_Click(object sender, EventArgs e)
        {

            computeNextGameState();
            panel1.Invalidate();
        }

        /*
         * World clock tick
         */
        private void timer1_Tick(object sender, EventArgs e)
        {
            computeNextGameState();
            panel1.Invalidate();
        }

        /*
         * User click on Play/Pause Button
         */
        private void buttonPlay_Click(object sender, EventArgs e)
        {
            if (timer1.Enabled)
            {
                timer1.Stop();
                buttonPlay.Text = "Play";
            }
            else
            {
                buttonPlay.Text = "Pause";
                timer1.Start();
            }
        }

        /*
        * Create or destroy a cell at click location
        */
        private void panel1_Click(object sender, EventArgs e)
        {
            CreateKillCell(((MouseEventArgs)e).Location);
            panel1.Invalidate(); // synchronize drawing with new world state
        }

        /*
         * User click on Reset Button
         */
        private void buttonReset_Click(object sender, EventArgs e)
        {
            ResetWorld();
            panel1.Invalidate();
        }

        /*
         * Perform all drawing operations here
         */ 
        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            // allocate buffered graphics => improved performances, less blinking
            BufferedGraphics bg = BufferedGraphicsManager.Current.Allocate(e.Graphics, e.ClipRectangle);
            // obtain graphics to draw into buffered graphics
            Graphics g = bg.Graphics;
            // clear graphics content
            g.Clear(Color.White);

            DrawWorld(g);

            bg.Render();
            bg.Dispose();
        }


        /********************************************
         * The Game !
         ********************************************/

        /*
        * Draw the entire world
        * Parameter : g : graphics of the component representing the world
        */
        private void DrawWorld(Graphics g)
        {
            DrawGrid(g);
            DrawCells(g);
        }

        /*
         * Draw the world grid : ie. horizontal and vertical lines delimiting each cell
         * Parameter : g : graphics of the component representing the world
         */
        private void DrawGrid(Graphics g)
        {
            Pen blackPen = new Pen(Color.Black);
            
            // !!!!!!!!!!!!!!!!!!    A MODIFIER ET COMPLETER !!!!!!!!!!!!!!!!!!

            // dessin des lignes verticales
            // Seules les 3 premières lignes sont dessinnées : utiliser une boucle for pour dessiner toutes les lignes verticales
            // 1ère ligne verticale
            g.DrawLine(blackPen, cellWidth * 0, 0, cellWidth * 0, cellWidth * worldHeight);
            // 2ème ligne verticale
            g.DrawLine(blackPen, cellWidth * 1, 0, cellWidth * 1, cellWidth * worldHeight);
            // 3ème ligne verticale
            g.DrawLine(blackPen, cellWidth * 2, 0, cellWidth * 2, cellWidth * worldHeight);

            // dessin des lignes horizontales
            // Seules les 3 premières lignes sont dessinnées : utiliser une boucle for pour dessiner toutes les lignes verticales
            // 1ère ligne horizontale
            g.DrawLine(blackPen, 0, cellWidth * 0, cellWidth * worldHeight, cellWidth * 0 );
            // 2ème ligne horizontale
            g.DrawLine(blackPen, 0, cellWidth * 1, cellWidth * worldHeight, cellWidth * 1);
            // 3ème ligne horizontale
            g.DrawLine(blackPen, 0, cellWidth * 2, cellWidth * worldHeight, cellWidth * 2);
        }

        /*
         * Draw all alive cells: ie. draw a square at a given position if cell is alive and leave blank otherwise
         * Parameter : g : graphics of the component representing the world
         */ 
        private void DrawCells(Graphics g)
        {
            
            Brush greenBrush = new SolidBrush(Color.Green);

            // !!!!!!!!!!!!!!!!!!    A MODIFIER ET COMPLETER !!!!!!!!!!!!!!!!!!

            // dessin de la première cellule : utiliser deux boucles for imbriquées pour dessiner toutes les cellules
            int i=0;
            int j=0;
            if (world[i, j]) // la cellule en position (i,j) est-elle en vie ?
                g.FillRectangle(greenBrush, cellWidth * i , cellWidth * j, cellWidth, cellWidth); // si oui on dessine
        }

   

        /*
         * Kill all living cells
         */ 
        private void ResetWorld()
        {
            // !!!!!!!!!!!!!!!!!!    A COMPLETER !!!!!!!!!!!!!!!!!!
        }

        /*
         * Convert coordinates from the x-y screen space (pixels) to the world coordinates
         * 
         * Parameter : Point p, position in the panel representing the world (coordinates in pixels)
         * Return : same position in the world coordinate
         * 
         * Precondition: p inside world
         */
        private Point ConvertPixelToWorldCoordinate(Point p)
        {
            Point position = new Point(0,0);

            // !!!!!!!!!!!!!!!!!!    A COMPLETER !!!!!!!!!!!!!!!!!!

            return position;
        }

        /*
         * Create or destroy a cell at click location
         * Parameter : Point p, click position in the panel representing the world (coordinates in pixels)
         * 
         */
        private void CreateKillCell(Point p)
        {

            // !!!!!!!!!!!!!!!!!!    A COMPLETER !!!!!!!!!!!!!!!!!!

        }

        /*
         * Count the number of living cells around position p
         * Parameter : Point p, position in world coordinates
         * Return : Number of living cells around the given position
         * 
         * Precondition: p inside world
         */  
        private int CountNeighbours(Point p)
        {
            int nb = 0;

            // !!!!!!!!!!!!!!!!!!    A COMPLETER !!!!!!!!!!!!!!!!!!

            return nb;
        }

        /*
         * Compute if cell at position p should be dead or alive at next round
         * Parameter: Point p, position in world coordinates
         * Return: The new state of the cell at the given position (false=>dead, true=>alive)
         * 
         * Precondition: p inside world
         */
        private bool ComputeNewCelluleState(Point p)
        {
            //Une cellule morte possédant exactement trois voisines vivantes devient vivante (elle naît).
            //Une cellule vivante possédant deux ou trois voisines vivantes le reste, sinon elle meurt.
            bool alive=false;

            // !!!!!!!!!!!!!!!!!!    A COMPLETER !!!!!!!!!!!!!!!!!!

            return alive;
        }

        /*
         * Compute the next state of the game
         */ 
        private void computeNextGameState()
        {
            bool[,] newWorld = new bool[worldHeight, worldWidth];

            // !!!!!!!!!!!!!!!!!!    A COMPLETER !!!!!!!!!!!!!!!!!!
            // le nouvel état des cellules doit être mis dans newWorld qui remplacera world à la fin de cette fonction.



            //
            world = newWorld; 

        }
    }
}
